{ FoxClone
  Copyright (C) 2019 - 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}

{-------------------------------------------------------------------------------
INCLUDE FILE BACKTAB.INC

Includes all procedures associated with the backup tab on the mainform.

Changes:

V1.0 - created 11 Jan 2021
-------------------------------------------------------------------------------}

{-------------------------------------------------------------------------------
PROCEDURE CREATEBACKUPPAGE
Creates the backup page objects - four boxes for source drive, source
partition, destination drive and destination partition, plus associated labels.
Two tbuttons, one to save and the other to quit.

Used by: the mainform constructor Create

V1.0 - 15 Oct 19
V1.1 - 30 Dec 19 - debug code
V1.2 - 17 Sep 20 - hint on partition checkbox changed to match revised partition info provided
                   by PartInfo.
-------------------------------------------------------------------------------}
procedure TMainFm.CreateBackupPage;
begin
  //debug
  if debug then DebugOut(true,'TMainFm.CreateBackupPage');
  //end debug
  {---------------------------------------------------------------------------
  Images and main title on the backup tab
  ---------------------------------------------------------------------------}
  DriveImg:=TImage.Create(Self);
  with DriveImg do begin
       Parent:=PageCtrl.Page[0];
       Left:= 550;
       Top:= 15;
       Picture.LoadFromFile(PartImgFile);
  end;
  FileImg:=TImage.Create(Self);
  with FileImg do begin
       Parent:=PageCtrl.Page[0];
       Left:= 700;
       Top:= 10;
       Picture.LoadFromFile(FileImgFile);
  end;
  ArrowImg:=TImage.Create(Self);
  with ArrowImg do begin
       Parent:=PageCtrl.Page[0];
       Left:= 635;
       Top:= 25;
       Picture.LoadFromFile(ArrowImgFile);
  end;
  //Main title for the tab
  BackTitleLbl := TLabel.Create(Self);
  with BackTitleLbl do
       begin
         Parent:= PageCtrl.Page[0];
         Top:= 20;
         Left:= LeftChkBox1;
         Width:= 150;
         Height:= 16;
         Caption:= 'Backup Partition(s) to Image File(s)';
         Font.Size:= 14;
         Show;
       end;
  {---------------------------------------------------------------------------
  Drive select tcheckbox, labels and event handlers
  ---------------------------------------------------------------------------}
  BackBox1Lbl := TLabel.Create(Self);
  with BackBox1Lbl do
       begin
         Parent := PageCtrl.Page[0];
         Top := TopChkBox1 - 20;
         Left := LeftChkBox1;
         Width := 150;
         Height := 16;
         Caption := '1 - Drive to Backup:';
         Show;
       end;
  //Drive select Tcheckbox
  BackChkBox1:= TCheckListBox.Create(Self);
  with BackChkBox1 do
       begin
         Parent:= PageCtrl.Page[0];
         Left := LeftChkBox1;
         Top := TopChkBox1;
         Width := ChkBoxWidth;
         Height := ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         //AllowGrayed:= true;
         //Color:= Yellow;
         ShowHint:= true;
         Hint:= DriveHint;
       end;
  //event handlers
  BackChkBox1.OnClickCheck:= @BackBox1CK;
  BackChkBox1.OnClick:= @BackBox1CK;
  {---------------------------------------------------------------------------
  Partition select tcheckbox, labels and event handlers
  ---------------------------------------------------------------------------}
  BackBox2Lbl := TLabel.Create(Self);
  with BackBox2Lbl do
       begin
         Parent := PageCtrl.Page[0];
         Top := TopChkBox1 - 20;
         Left := LeftChkBox2;
         Width := 150;
         Height := 16;
         Caption := '2 - Partition(s) to Backup:';
         Show;
       end;
  BackChkBox2:= TCheckListBox.Create(Self);
  with BackChkBox2 do
       begin
         Parent:= PageCtrl.Page[0];
         Left:= LeftChkBox2;
         Top:= TopChkBox1;
         Width:= ChkBoxWidth;
         Height:= ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         ShowHint:= true;
         Hint:= PartitionHint + LF + 'Multiple partitions can be selected';
         Enabled:= false;
       end;
  //event handlers
  BackChkBox2.OnClickCheck:= @BackBox2CK;
  BackChkBox2.OnClick:= @BackBox2CK;
  {---------------------------------------------------------------------------
  Destination select tcheckbox, labels and event handlers
  ---------------------------------------------------------------------------}
  BackBox3Lbl:= TLabel.Create(Self);
  with BackBox3Lbl do
       begin
         Parent:= PageCtrl.Page[0];
         Top:= TopChkBox2 - 20;
         Left:= LeftChkBox1;
         Width:= 150;
         Height:= 16;
         Caption:= '3 - Destination Drive:';
         Show;
       end;
  BackChkBox3:= TCheckListBox.Create(Self);
  with BackChkBox3 do
       begin
         Parent:= PageCtrl.Page[0];
         Left:= LeftChkBox1;
         Top:= TopChkBox2;
         Width:= ChkBoxWidth;
         Height:= ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         ShowHint:= true;
         Hint:= DriveHint;
         Enabled:= false;
       end;
  //event handlers
  BackChkBox3.OnClickCheck:= @BackBox3CK;
  BackChkBox3.OnClick:= @BackBox3CK;
  {---------------------------------------------------------------------------
  Destination partition select tcheckbox, labels and event handlers
  ---------------------------------------------------------------------------}
  BackBox4Lbl := TLabel.Create(Self);
  with BackBox4Lbl do
       begin
         Parent:= PageCtrl.Page[0];
         Top:= TopChkBox2 - 20;
         Left:= LeftChkBox2;
         Width:= 150;
         Height:= 16;
         Caption:= '4 - Destination Partition:';
         Show;
      end;
  BackChkBox4:= TCheckListBox.Create(Self);
  with BackChkBox4 do
       begin
         Parent:= PageCtrl.Page[0];
         Left:= LeftChkBox2;
         Top:= TopChkBox2;
         Width:= ChkBoxWidth;
         Height:= ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         ShowHint:= true;
         Hint:= PartitionHint + LF + 'Only a single partition can be selected';
         Enabled:= false;
       end;
  //event handlers
  BackChkBox4.OnClickCheck:= @BackBox4CK;
  BackChkBox4.OnClick:= @BackBox4CK;
  {---------------------------------------------------------------------------
  File save button
  ---------------------------------------------------------------------------}
  NextBtn:= TButton.Create(Self);
  with NextBtn do
       begin
         Parent:= PageCtrl.Page[0];
         Width:= 80;
         Left:= LeftChkBox2 + ChkBoxWidth - Width;
         Top:= TopChkBox2 + ChkBoxHeight + 20;
         Height:= 30;
         Show;
         Caption:= 'Save to File';
         Enabled:= false; //turn off until needed
         OnClick := @BackSaveCK;
       end;
  {---------------------------------------------------------------------------
  Quit button to terminate
  ---------------------------------------------------------------------------}
  QuitBtn:= TButton.Create(Self);
  with QuitBtn do
       begin
         Parent:= PageCtrl.Page[0];
         Width:= 80;
         Left:= LeftChkBox2 + ChkBoxWidth - 2 * Width - 10;
         Top:= TopChkBox2 + ChkBoxHeight + 20;
         Height:= 30;
         Show;
         Caption:= 'Quit';
         Enabled:= true; //turn off until needed
         OnClick := @QuitCK;
         Visible:= true; //not visible until needed
       end;
end; //end procedure createbackuppage
{-------------------------------------------------------------------------------
PROCEDURE BACKBOX1CK
This is the event handler for the source drive check list box.

Used by: CreateBackupPage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.BackBox1CK(Sender: TObject);
begin
  if BackChkBox1.ItemIndex >= 0 then BackBox1(BackChkBox1.ItemIndex)
end; //end procedure backbox1ck
{-------------------------------------------------------------------------------
PROCEDURE BACKBOX2CK
This is the event handler for the source partition check list box.

Used by: CreateBackupPage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.BackBox2CK(Sender: TObject);
begin
  if BackChkBox2.ItemIndex >= 0 then BackBox2(BackChkBox2.ItemIndex);
end; //end procedure backbox2ck
{-------------------------------------------------------------------------------
PROCEDURE BACKBOX3CK
This is the event handler for the destination drive check list box.

Used by: CreateBackupPage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.BackBox3CK(Sender: TObject);
begin
  if BackChkBox3.ItemIndex >= 0 then BackBox3(BackChkBox3.ItemIndex);
end; //end procedure backbox3ck
{-------------------------------------------------------------------------------
PROCEDURE BACKBOX4CK
This is the event handler for the destination partition check list box.
It is triggered by both the onclick and oncheck events.
Checks the partition type - cannot save to unknown filesystems, or extended
partitions. Checks if enough space for the backup on the destination partition.
Enables the save button.

Used by: CreateBackupPage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.BackBox4CK(Sender: TObject);
begin
  if BackChkBox4.ItemIndex >= 0 then BackBox4(BackChkBox4.ItemIndex);
end; //end procedure backbox4ck
{-------------------------------------------------------------------------------
PROCEDURE BACKSAVECK
Event handler for the save button on the backup page.
Opens the save dialog and prompts for a file to save to.
Creates and displays the backup confirm form.

Used by: CreateBackupPage

V1.0 - 15 Oct 2019
V1.1 - 07 Jan 2020 - debug code
V1.2 - 29 Feb 2020 - added validfile check to see if backup can be saved to destination
V1.3 - 03 Mar 2020 - on the save dialog the user could select another drive/partition as the destination
                     for a backup. This is now checked and a messagebox opened if it happens.
                     Validfile removed - no longer needed, running as root.
V1.4 - 09 Mar 2020 - coding error in v1.3 meant that foxclone was unmounting any partition it had to
                     mount BEFORE doing the backup = failure. Reason - backconfirm calls the confform
                     and then the prog form and neither are showmodal (and can't be) so code execution
                     continues past them to the end of this procedure.
-------------------------------------------------------------------------------}
procedure TMainFm.BackSaveCK(Sender: TObject);
var
  SaveDialog : TSaveDialog;
  today : string; //today's date in format YYYYMMDD
  i : integer;
  mountpoint : string;
  MountOK : boolean = false;
  ForceMount : boolean = false;
  UserExit : boolean = false;
begin
  i:= 1;
  //find the destination partition
  while not DestDrive.parts[i].selected do i+= 1;
  mountpoint:= CheckMount(Destdrive.parts[i].pnumber);
  //debug
  if debug then DebugOut(true,'TMainFm.BackSaveCK','selected destination',Destdrive.parts[i].pnumber,
                         'mount point',mountpoint);
  //end debug
  if mountpoint <> '<none>' then MountOK:= true
  else //not mounted so mount
     if mkdir(Destdrive.parts[i].pnumber) = 0 then //folder for mount created
        if mount(Destdrive.parts[i].pnumber,true) = 0 then //partition mounted
           begin
             mountpoint:= MountFolder + Destdrive.parts[i].pnumber;
             MountOK:= true;
             ForceMount:= true;
           end;
  if MountOK then
     begin
       //get today's date as starting point for backup filename, format YYYYMMDD
       today:= IntToStr(YearOf(now));
       if MonthOf(now) < 10 then today+= '0' + IntToStr(MonthOf(now))
          else today+= IntToStr(MonthOf(now));
       if DayOf(now) < 10 then today+= '0' + IntToStr(DayOf(now))
          else today+= IntToStr(DayOf(now));
       BackFileName:= '';
       //open save dialog for file name and location
       SaveDialog:= TSaveDialog.Create(Self);
       SaveDialog.Filename:= today;
       SaveDialog.InitialDir:= mountpoint;
       if SaveDialog.Execute then
          //the user may have selected a different drive/partition from the left hand panel on the
          //save dialog. This must be trapped.
          if pos(mountpoint,SaveDialog.Filename) > 0 then
             begin
               BackFileName:= SaveDialog.Filename;
               BackConfirm;
             end
          else
            begin
              MessageDlg('Destination changed',
                         'Destination drive/partition must be the same as selected in the main window',
                         mtError, [mbOK], 0);
              UserExit:= true;
            end
       else UserExit:= true; //the user has cancelled
       if ForceMount and UserExit then
           begin
             umount(Destdrive.parts[i].pnumber);
             rmdir(Destdrive.parts[i].pnumber);
           end;
       SaveDialog.Free;
     end;
end;
{-------------------------------------------------------------------------------
PROCEDURE QUITCK
Closes the programme

Used by: CreateBackupPage, CreateRestorePage, CreateClonePage, CreateSettingsPage

V1.0 - 15 Oct 2019
V1.1 - 23 Jan 2020, problem with the iso build with a 5.3 kernel, it will not shutdown properly. This has
                    been fixed by changing shutdown to halt, but immediately terminates. QuitCK amended
                    so that is checks all partitions are unmounted before closing foxclone.
                    Subsequent checking - using the halt command in the iso does umount all drives.
V1.2 - 09 Mar 2020, was unmounting any partitions mounted but NOT removing the mount folder
-------------------------------------------------------------------------------}
procedure TMainFm.QuitCK(Sender: TObject);
var
  i,j : integer;
  Mount : string;
begin
  //check and unmount all partitions
  for i:= 1 to TotalDrives do with MyDrives[i] do
      for j:= 1 to totalparts do
        begin
          Mount:= CheckMount(parts[j].pnumber);
          //if a mount point is root then foxclone is not running from an iso and has been
          //installed inside a working linux system
          if pos(MountFolder,Mount) > 0 then
             begin
               umount(parts[j].pnumber);
               rmdir(parts[j].pnumber);
             end;
        end;
  //close
  if Assigned(ConfirmForm) then ConfirmForm.Free;
  if Assigned(ProgressFm) then ProgressFm.Free;
  application.Terminate;
end;

