{ FoxClone
  Copyright (C) 2019 - 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}

{-------------------------------------------------------------------------------
INCLUDE FILE CLONETAB.INC

Includes all procedures associated with the clone tab on the mainform.

Changes:

V1.0 - created 11 Jan 2021
-------------------------------------------------------------------------------}

{-------------------------------------------------------------------------------
PROCEDURE CREATECLONEPAGE
Creates the clone page objects - a combo dropdown (either clone drive to drive
or backup file to drive), two boxes for source drive and target drive, and
two optional boxes for file to drive, one showing the contents of the backup file
and the other, the current contents of the target drive.
Two tbuttons, one to clone and the other to quit. There is also a note button
which is enabled if there is a note associated with the backup.

Used by: the mainform constructor Create

V1.0 - 15 Oct 19
V1.1 - 30 Dec 19 - debug code
-------------------------------------------------------------------------------}
procedure TMainFm.CreateClonePage;
begin
  //debug
   if debug then DebugOut(true,'TMainFm.CreateClonePage');
  //end debug
  {---------------------------------------------------------------------------
  Images and main title on the restore tab
  ---------------------------------------------------------------------------}
  CSourceImg:=TImage.Create(Self);
  with CSourceImg do begin
       Parent:=PageCtrl.Page[2];
       Left:= 540;
       Top:= 10;
       Picture.LoadFromFile(DriveImgFile);
  end;
  CDestImg:=TImage.Create(Self);
  with CDestImg do begin
       Parent:=PageCtrl.Page[2];
       Left:= 700;
       Top:= 10;
       Picture.LoadFromFile(DriveImgFile);
  end;
  ArrowImg:=TImage.Create(Self);
  with ArrowImg do begin
       Parent:=PageCtrl.Page[2];
       Left:= 635;
       Top:= 25;
       Picture.LoadFromFile(ArrowImgFile);
  end;
  CloneLbl := TLabel.Create(Self);
  with CloneLbl do
       begin
         Parent:= PageCtrl.Page[2];
         Top:= 20;
         Left:= LeftChkBox1;
         Width:= 150;
         Height:= 16;
         Caption:= 'Clone';
         Font.Size:= 14;
         Show;
       end;
  {---------------------------------------------------------------------------
  Combo dropdown - drive to drive or backup file to drive
  ---------------------------------------------------------------------------}
  CloneComboBox:= TComboBox.Create(Self);
  with CloneComboBox do
       begin
         Parent:= PageCtrl.Page[2];
         Left:= LeftChkBox1 + 80;
         Top:= CloneLbl.Top + 2;
         Width:= 140;
         Height:= 20;
         Style:= csDropDownList;
         Items.Add('Drive to Drive');
         Items.Add('Backup file to Drive');
         ItemIndex:= 0;
         ShowHint:= true;
         Hint:= 'A direct copy drive to drive,' + LF + 'or from a backup file to drive';
         OnChange := @CloneType;
       end;
  {---------------------------------------------------------------------------
  Source drive (or source for backup file) checkbox, labels and event handlers
  ---------------------------------------------------------------------------}
   ClCheckBox1Lbl := TLabel.Create(Self);
   with ClCheckBox1Lbl do
        begin
          Parent := PageCtrl.Page[2];
          Top := TopChkBox1 - 20;
          Left := LeftChkBox1;
          Width := 150;
          Height := 16;
          Caption := '1 - Source Drive:';
          Show;
        end;
  ClChkBox1:= TCheckListBox.Create(Self);
  with ClChkBox1 do
       begin
         Parent:= PageCtrl.Page[2];
         Left := LeftChkBox1;
         Top := TopChkBox1;
         Width := ChkBoxWidth;
         Height := ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         ShowHint:= true;
         Hint:= 'Mount point' + #13 + 'Model name'
                 + #13 + 'Drive size' + #13 + 'Partition table type' + #13
                 + 'Only a single drive can be selected';
       end;
  //event handlers
  ClChkBox1.OnClickCheck:= @CloneBox1CK;
  ClChkBox1.OnClick:= @CloneBox1CK;
  {---------------------------------------------------------------------------
  Target drive (or source partition for backup file) checkbox, labels & event handlers
  ---------------------------------------------------------------------------}
  ClCheckbox2Lbl := TLabel.Create(Self);
  with ClCheckbox2Lbl do
       begin
         Parent := PageCtrl.Page[2];
         Top := TopChkBox1 - 20;
         Left := LeftChkBox2;
         Width := 150;
         Height := 16;
         Caption:= '2 - Target Drive';
         Show;
       end;
  ClChkbox2:= TCheckListBox.Create(Self);
  with ClChkbox2 do
       begin
         Parent:= PageCtrl.Page[2];
         Left:= LeftChkBox2;
         Top:= TopChkBox1;
         Width:= ChkBoxWidth;
         Height:= ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         Enabled:= false;
       end;
  //event handlers
  ClChkbox2.OnClickCheck:= @CloneBox2CK;
  ClChkbox2.OnClick:= @CloneBox2CK;
  {---------------------------------------------------------------------------
  File to drive only - content of backup file checkbox, labels & event handlers
  ---------------------------------------------------------------------------}
  ClListbox3Lbl:= TLabel.Create(Self);
  with ClListbox3Lbl do
       begin
         Parent:= PageCtrl.Page[2];
         Top:= TopChkBox2 - 20;
         Left:= LeftChkBox1;
         Width:= 150;
         Height:= 16;
         Caption:= '3 - Contents of clone file';
         Visible:= false;
       end;
  ClListbox3:= TListBox.Create(Self);
  with ClListbox3 do
       begin
         Parent:= PageCtrl.Page[2];
         Left:= LeftChkBox1;
         Top:= TopChkBox2;
         Width:= ChkBoxWidth;
         Height:= ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         ShowHint:= true;
         Hint:= 'Mount point' + LF + 'Model name'
                 + LF + 'Drive size' + LF + 'Partition table type' + LF
                 + 'Only a single drive can be selected';
          Enabled:= false;
          Visible:= false;
       end;
  ClDateLbl := TLabel.Create(Self);
  with ClDateLbl do
       begin
         Parent := PageCtrl.Page[2];
         Top := TopChkBox2 + ChkBoxHeight + 20;
         Left := LeftChkBox1;
         Width := 150;
         Height := 16;
         Caption := '';
         Visible:= false;
        end;
  {---------------------------------------------------------------------------
  Target drive checkbox, labels & event handlers
  ---------------------------------------------------------------------------}
  ClCheckBox4Lbl := TLabel.Create(Self);
  with ClCheckBox4Lbl do
       begin
         Parent:= PageCtrl.Page[2];
         Top:= TopChkBox2 - 20;
         Left:= LeftChkBox2;
         Width:= 150;
         Height:= 16;
         Caption:= '4 - Target Drive:';
         Visible:= false;
      end;
  //Destination partition select Tcheckbox
  ClChkBox4:= TCheckListBox.Create(Self);
  with ClChkBox4 do
       begin
         Parent:= PageCtrl.Page[2];
         Left:= LeftChkBox2;
         Top:= TopChkBox2;
         Width:= ChkBoxWidth;
         Height:= ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         Visible:= false;
         Enabled:= false;
       end;
  //event handlers
  ClChkBox4.OnClickCheck:= @CloneBox4CK;
  ClChkBox4.OnClick:= @CloneBox4CK;
  {---------------------------------------------------------------------------
  Note button - enabled if backup file has a note attached
  was able to get away with a single quit button, because it is always visible
  and always does the same thing, but not the note button. Clone and restore
  switch this button on/off and if only a single button, it happens to the last
  Note button created = the one on the clone page. Hence a separate note button
  although it does exactly the same thing on both pages.
  ---------------------------------------------------------------------------}
  CloneNoteBtn:= TButton.Create(Self);
  with CloneNoteBtn do
       begin
         Parent:= PageCtrl.Page[2];
         Width:= 80;
         Left:= LeftChkBox1 + RestChkBox3.Width - 80;
         Top:= TopChkBox2 + ChkBoxHeight + 20;
         Height:= 30;
         Visible:= false;
         Caption:= 'View Note';
         Enabled:= false; //turn off until needed
         OnClick := @RestNoteCK;
       end;
  {---------------------------------------------------------------------------
  Clone button
  ---------------------------------------------------------------------------}
  CloneBtn:= TButton.Create(Self);
  with CloneBtn do
       begin
         Parent:= PageCtrl.Page[2];
         Width:= 80;
         Left:= LeftChkBox2 + ChkBoxWidth - Width;
         Top:= TopChkBox2 + ChkBoxHeight + 20;
         Height:= 30;
         Show;
         Caption:= 'Clone';
         Enabled:= false; //turn off until needed
         OnClick := @CloneBtnCK;
       end;
  {---------------------------------------------------------------------------
  Quit button to terminate
  ---------------------------------------------------------------------------}
  QuitBtn:= TButton.Create(Self);
  with QuitBtn do
       begin
         Parent:= PageCtrl.Page[2];
         Width:= 80;
         Left:= LeftChkBox2 + ChkBoxWidth - 2 * Width - 10;
         Top:= TopChkBox2 + ChkBoxHeight + 20;
         Height:= 30;
         Show;
         Caption:= 'Quit';
         Enabled:= true; //turn off until needed
         OnClick := @QuitCK;
         Visible:= true; //not visible until needed
       end;
end; //end procedure TMainFm.CreateClonePage
{-------------------------------------------------------------------------------
PROCEDURE CLONETYPE
Event handler for the combo drop down which is either disk to disk clone or
clone from an existing backup to disk.  Basically sets all the checkboxes and
labels back to their default states. This varies dependent on whether the clone
is disk to disk or file to disk.

Used by: CreateClonePage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.CloneType(Sender: TObject);
begin
  //Reset everything
  ClChkBox1.CheckAll(cbUnchecked);
  ClChkBox2.Clear;
  ClChkBox2.Enabled:= false;
  ClListBox3.Clear;
  ClListBox3.Enabled:= false;
  ClDateLbl.Visible:= false;
  NoteBtn.Enabled:= false;
  NoteBtn.Visible:= false;
  ClListBox3.ScrollWidth:= ClListBox3.Width;
  CloneBtn.Enabled:= false;
  ClChkBox4.Enabled:= false;
  ClChkBox4.Clear;
  case CloneComboBox.ItemIndex of
       0 : begin //Drive to drive clone
             CSourceImg.Picture.LoadFromFile(DriveImgFile);
             CDestImg.Picture.LoadFromFile(DriveImgFile);
             ClCheckbox1Lbl.Caption:= '1 - Source Drive';
             ClCheckbox2Lbl.Caption:= '2 - Target Drive';
             ClListbox3Lbl.Visible:= false;
             ClListBox3.Visible:= false;
             ClCheckbox4Lbl.Visible:= false;
             ClChkBox4.Visible:= false;
           end;
       1 : begin //File to drive clone
             CSourceImg.Picture.LoadFromFile(FileImgFile);
             CDestImg.Picture.LoadFromFile(DriveImgFile);
             ClCheckbox1Lbl.Caption:= '1 - Drive containing clone files';
             ClCheckbox2Lbl.Caption:= '2 - Partition containing clone files';
             ClListbox3Lbl.Visible:= true;
             ClListBox3.Visible:= true;
             ClCheckbox4Lbl.Visible:= true;
             ClChkBox4.Visible:= true;
           end;
  end;
end; //end procedure TMainFm.CloneType
{-------------------------------------------------------------------------------
PROCEDURE CLONEBOX1CK
Event handler for first check box. This is the source drive to clone or the
drive containing the backup file to clone.

Used by: CreateClonePage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.CloneBox1CK(Sender: TObject);
begin
  if ClChkBox1.ItemIndex >= 0 then
     case CloneComboBox.ItemIndex of
          0 : CloneDDrive(ClChkBox1.ItemIndex);
          1 : CloneDFile(ClChkBox1.ItemIndex);
     end;
end; //end procedure TMainFm.CloneBox1CK
{-------------------------------------------------------------------------------
PROCEDURE CLONEBOX2CK
Event handler for second check box. This will be either the target drive or the
partition on the source drive containing the backup file to clone.
If the clone is drive to drive, check that there is enough space on the target
drive.
If the clone is file to drive, call the procedure to get the backup file.

Used by: CreateClonePage

V1.0 - 15 Oct 2019
V1.1 - 20 Mar 2020, check added to stop the user selecting the boot drive as the target for cloning
-------------------------------------------------------------------------------}
procedure TMainFm.CloneBox2CK(Sender: TObject);
var
  Source : integer;
  Target : string;
begin
  if ClChkBox2.ItemIndex >= 0 then
     begin
       //only one item can be checked
       Source:= ClChkBox2.ItemIndex;
       ClChkBox2.CheckAll(cbUnchecked);
       ClChkBox2.Checked[Source]:= true;
       case CloneComboBox.ItemIndex of
            0 : begin
                  Target:= trim(copy(ClChkBox2.Items[Source],1,pos(':',ClChkBox2.Items[Source])-1));
                  //check if the foxclone boot drive has been selected
                  if Target <> BootDrive then
                     begin
                       ClChkBox2.Checked[Source]:= CloneCheck(CloneDrive.mountpt,Target,false);
                       CloneBtn.Enabled:= ClChkBox2.Checked[Source];
                     end
                  else
                     begin
                       CloneBtn.Enabled:= false;
                       ClChkBox2.Checked[Source]:= false;
                       MessageDlg('Boot Drive','Cannot clone to ' + MyDrives[Source + 1].mountpt + '.'
                                  + LF + 'It is the foxclone boot drive.',mtError,[mbOK],0);
                     end;
                end;
            1 : CloneFileOpen(ClChkBox2.ItemIndex);
       end;
     end;
end; //end procedure TMainFm.CloneBox2CK
{-------------------------------------------------------------------------------
PROCEDURE CLONEBOX4CK
Event handler for checkbox4. This is only used in a file to drive clone. The
checkbox displays the target drive. Checks if there is enough space on the
target drive for the clone.

Used by: CreateClonePage

V1.0 - 15 Oct 2019
V1.1 - 20 Mar 2020, check added so that the user does not select the foxclone boot drive as the target.
-------------------------------------------------------------------------------}
procedure TMainFm.CloneBox4CK(Sender: TObject);
var
  Source : integer;
  Target : string;
  SourceFile : string;
begin
  if ClChkBox4.ItemIndex >= 0 then
     begin
       //only one item can be checked
       Source:= ClChkBox4.ItemIndex;
       ClChkBox4.CheckAll(cbUnchecked);
       //CloneCheck(Source : string; TargetDrive : string; sfFile : boolean) : boolean;
       SourceFile:= ReplaceStr(RestFileName,'.backup','.sfdisk');
       Target:= trim(copy(ClChkBox4.Items[Source],1,pos(':',ClChkBox4.Items[Source])-1));
       //check that target is not the foxclone boot drive
       if Target <> BootDrive then
          begin
            ClChkBox4.Checked[Source]:= CloneCheck(SourceFile,Target,true);
            CloneBtn.Enabled:= ClChkBox4.Checked[Source];
          end
       else
          begin
            ClChkBox4.Checked[Source]:= false;
            CloneBtn.Enabled:= false;
            MessageDlg('Boot Drive','Cannot clone to ' + Target + '.'
                       + LF + 'It is the foxclone boot drive.',mtError,[mbOK],0)
          end;
     end;
end; //end procedure TMainFm.CloneBox4CK
{-------------------------------------------------------------------------------
PROCEDURE CLONEBTNCK
Event handler the clone button.

Used by: CreateClonePage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.CloneBtnCK(Sender: TObject);
begin
  case CloneComboBox.ItemIndex of
       0 : CloneDiskConfirm;
       1 : CloneFileConfirm;
  end;
end; //end procedure TMainFm.CloneBtnCK

