{ FoxClone
  Copyright (C) 2019 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}

{-------------------------------------------------------------------------------
UNIT PROGFORM
This is the progress window that displays when doing a backup, restore or clone.

Changes:

V1.0 - 06 Jun 2019
V1.1 - 16 Jan 2021, view log button added
-------------------------------------------------------------------------------}
unit progform;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, ComCtrls, StdCtrls, Graphics, Dialogs,
  utilities, verify, clone;

type
  TProgressFm = class(TForm)
    PartcloneLbl: TLabel;
    CurrentProgBar : TProgressBar;
    CurrentProgLbl : TLabel;
    OverallProgBar : TProgressBar;
    OverallProgLbl : TLabel;
    PartListBox : TListBox;
    CancelBtn : TButton;
    FinishBtn : TButton;
    AddBtn : TButton;
    LogBtn : TButton;

  public
    constructor Create(TheOwner: TComponent);
    procedure ProgFmFinish(Sender: TObject);
    procedure ProgFmCancel(Sender: TObject);
    procedure ProgFmAddNote(Sender: TObject);
    procedure ProgFmViewLog(Sender: TObject);
end;

var
 CancelPartClone : boolean = false;
 ProgressFm: TProgressFm;

implementation
uses noteform, mainform;
{$R *.lfm}
{-------------------------------------------------------------------------------
CONSTRUCTOR
Creates the progress window.

Used by: unit restore - RestRestore
         unit clone - CloneDiskToDisk, CloneFileToDisk
         unit backup - BackBackup

V1.0 - 05 Jun 2019
V1.1 - 24 Sep 2020 : BorderStyle added to stop window resizing
V1.2 - 16 Jan 2021 : Log button added
-------------------------------------------------------------------------------}
constructor TProgressFm.Create(TheOwner: TComponent);
const
  bHeight : longint = 270;  //main window initial height
  bWidth : longint = 450;   //main window initial width
  Lmargin = 20; //left margin for objects in the main window
  LF = LineEnding;
begin
  inherited CreateNew(TheOwner,1);
  BorderStyle:= bsSingle;
  Font.Name:= 'Noto Sans';
  Font.Size:= 10;
  Caption:= 'Progress'; //caption for the title bar
  Position:=poScreenCenter;
  SetInitialBounds(0,0,bWidth,bHeight); //defines initial size
  Visible:= true;
  PartcloneLbl:= TLabel.Create(ProgressFm);
  with PartcloneLbl do
       begin
         parent:= Self;
         Width:= 200;
         Left:= Lmargin;
         Top:= 20;
         //Top:= PartNumber.Top + 30;
         Height:= 20;
         Caption:= 'File system:' + LF + 'Device Size:' + LF + 'Space in use:';
       end;
  CurrentProgBar:= TProgressBar.Create(ProgressFm);
  with CurrentProgBar do
       begin
         Parent := Self;
         Left := Lmargin;
         Top  := PartcloneLbl.Top + 60;
         Width:= 410;
         Height:= 12;
         BarShowText := false;
         Smooth := True;
         Min:= 0;
         Position := 0;
       end;
  CurrentProgLbl:= TLabel.Create(ProgressFm);
  with CurrentProgLbl do
       begin
         Parent:= Self;
         Width:= 200;
         Font.Size:= 9;
         Left:= Lmargin;
         Top:= CurrentProgBar.Top + 15;
         Height:= 30;
         Caption:= 'Calculating bitmap, please wait...';
       end;
  OverallProgBar:= TProgressBar.Create(ProgressFm);
  with OverallProgBar do
       begin
         Parent := Self;
         Left := Lmargin;
         Top  := CurrentProgLbl.Top + 30;
         Width:= 410;
         Height:= 12;
         BarShowText := false;
         Smooth := True;
         Min:= 0;
         Position := 0;
       end;
  OverallProgLbl:= TLabel.Create(ProgressFm);
  with OverallProgLbl do
       begin
         parent:= Self;
         Width:= 200;
         Font.Size:= 9;
         Left:= Lmargin;
         Top:= OverallProgBar.Top + 15;
         Height:= 30;
         Caption:= 'overall progress label...';
       end;
  PartListBox:= TListBox.Create(ProgressFm);
  with PartListBox do
       begin
         parent:= Self;
         Width:= 200;
         ItemHeight:= 12;
         Style:=lbOwnerDrawVariable; // required for itemheight to work
         Left:= Lmargin;
         Top:= OverallProgLbl.Top + 30;
         Height:= 80;
       end;
  CancelBtn:= TButton.Create(ProgressFm);
  with CancelBtn do
       begin
         parent:= Self;
         Width:= 80;
         Left:= 260;
         Top:= OverallProgLbl.Top + 80;
         Height:= 30;
         Show;
         Caption:= 'Cancel';
         OnClick := @ProgFmCancel;
        end;
  FinishBtn:= TButton.Create(ProgressFm);
  with FinishBtn do  //button to start partclone
       begin
         parent:= Self;
         Width:= 80;
         Left:= 350;
         Top:= OverallProgLbl.Top + 80;
         Height:= 30;
         Show;
         Caption:= 'Finish';
         Enabled:= false; //turn off until needed
         OnClick := @ProgFmFinish;
       end;
  AddBtn:= TButton.Create(ProgressFm);
  with AddBtn do  //button to start partclone
       begin
         parent:= Self;
         Width:= 80;
         Left:= 350;
         Top:= OverallProgLbl.Top + 40;
         Height:= 30;
         Show;
         Caption:= 'Add Note';
         Enabled:= false; //turn off until needed
         OnClick := @ProgFmAddNote;
        end;
  LogBtn:= TButton.Create(ProgressFm);
  with LogBtn do  //button to start partclone
       begin
         parent:= Self;
         Width:= 80;
         Left:= 350;
         Top:= OverallProgLbl.Top + 35;
         Height:= 30;
         Show;
         Caption:= 'View Log';
         Enabled:= false; //turn off until needed
         Visible:= false;
         OnClick := @ProgFmViewLog;
        end;

end;//end constructor
{-------------------------------------------------------------------------------
PROCEDURE - TProgressFm.PROGFMFINISH

Event handler for the finish button on the progress form. Closes the form.

Used by: progform constructor

V1.0 - 05 Jun 2019
V1.1 - 23 Feb 2020 : amended to call readdrives after a restore a clone to update foxclone
-------------------------------------------------------------------------------}
procedure TProgressFm.ProgFmFinish(Sender: TObject);
begin
  Close;
  if not (CurrentAction = acBack) then
     begin
       FormActivated:= false;
       MainFm.ReadDrives(MainFm);
       //need to reset checkboxes
       case CurrentAction of
         acRest      : with MainFm do
                            begin
                              RestChkBox2.Enabled:= false;
                              RestChkBox2.Clear;
                              RestChkBox3.Clear;
                              RestChkBox3.Enabled:= false;
                              RestChkBox3.ScrollWidth:= ChkBoxWidth;
                              RestBox3Lbl.Caption:= '3 - Contents of Backup:';
                              RestLstBox4.Clear;
                              RestLstBox4.Enabled:= false;
                              RestLstBox4.ScrollWidth:= ChkBoxWidth;
                              RestBox4Lbl.Caption:= '4 - Destination Drive:';
                              NoteBtn.Enabled:= false;
                              NoteBtn.Visible:= false;
                              RestBtn.Enabled:= false;
                              RDestInfoLbl.Visible:= false;
                              RDateLbl.Visible:= false;
                            end;
         acClone     : with MainFm do
                            begin
                              ClChkBox2.Clear;
                              ClChkBox2.Enabled:= false;
                              CloneBtn.Enabled:= false;
                            end;
         acFileClone : with MainFM do
                            begin
                              ClChkBox2.Clear;
                              ClChkBox2.Enabled:= false;
                              ClListBox3.Enabled:= false;
                              ClListBox3.Clear;
                              ClListBox3.ScrollWidth:= ChkBoxWidth;
                              ClListbox3Lbl.Caption:= '3 - Contents of clone file';
                              ClChkBox4.Enabled:= false;
                              ClChkBox4.Clear;
                              CloneBtn.Enabled:= false;
                              CloneNoteBtn.Enabled:= false;
                              CloneNoteBtn.Visible:= false;
                              ClDateLbl.Visible:= false;
                            end;
       end; //case
     end;
  MainFm.PageCtrl.Enabled:= true;
end; //end procedure TProgressFm.ProgFmFinish

{-------------------------------------------------------------------------------
PROCEDURE - TProgressFm.PROGFMCANCEL

Event handler for the cancel button on the backup form. Closes the form.

V1.1 - found that repeatedly clicking the cancel button could generate an error so disable the
cancel button once clicked. Doesn't make any difference, the button is not greyed out. Also getting
message that can't kill partclone cos no process found?

Used by: progform constructor

V1.0 - 05 Jun 2019
v1.1 - 30 Oct 2019
v1.2 - 18 Jan 2021 - amended for cancelling during a verify
-------------------------------------------------------------------------------}
procedure TProgressFm.ProgFmCancel(Sender: TObject);
begin
  CancelBtn.Enabled:= false;
  application.processmessages;
  if currentaction = acBack then
     MessageDlg('User Abort','Any backup files created should be manually deleted', mtWarning,[mbOK],0)
  else MessageDlg('User Abort','Any *.chk files in the backup folder can be deleted', mtInformation,[mbOK],0);
  CancelPartClone:= true;
end; //end procedure TProgressFm.ProgFmCancel
{-------------------------------------------------------------------------------
PROCEDURE - TProgressFm.PROGFMADDNOTE

Event handler for the add note button on the progesss form.

Used by: progform constructor

V1.0 - 05 Jun 2019
-------------------------------------------------------------------------------}
procedure TProgressFm.ProgFmAddNote(Sender: TObject);
var
  NoteForm: TNoteForm;
begin
  NoteForm:= TNoteForm.Create(ProgressFm);
  NoteForm.Show;
 end; //end procedure TProgressFm.ProgFmAddNote
{-------------------------------------------------------------------------------
PROCEDURE - TProgressFm.PROGFMVIEWLOG

Event handler for the view log button on the progress form.

Used by: progform constructor

V1.0 - 16 Jan 2021
-------------------------------------------------------------------------------}
procedure TProgressFm.ProgFmViewLog(Sender: TObject);
begin
  case CurrentAction of
    acVerify : writeln('current action = acverify');
    acBack  : writeln('current action = acback');
    acRest: writeln('current action = acrest');
    acClone : writeln('current action = acclone');
    acFileClone : writeln('current action = acfileclone');
  end;
  if CurrentAction = acVerify then VerifyLog else CloneLog;
end; //end procedure TProgressFm.ProgFmViewLog

end.
