{ FoxClone
  Copyright (C) 2019 - 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}

{-------------------------------------------------------------------------------
INCLUDE FILE RESTTAB.INC

Includes all procedures associated with the restore tab on the mainform.

Changes:

V1.0 - created 11 Jan 2021
-------------------------------------------------------------------------------}

{-------------------------------------------------------------------------------
PROCEDURE CREATRESTOREPAGE
Creates the restore page objects - four boxes for source drive (which drive is the
backup file on, source partition (which partition is the backup file in),
destination drive (contents of the backup) and destination drive (current content
of destination drive), plus associated labels.
Two tbuttons, one to restore and the other to quit. There is also a note button
which is enabled if there is a note associated with the backup.

Used by: the mainform constructor Create

V1.0 - 15 Oct 19
V1.2 - 30 Dec 19, debug code
V1.3 - 20 Sep 20, hints added to check/listboxes
-------------------------------------------------------------------------------}
procedure TMainFm.CreateRestorePage;
begin
  //debug
  if debug then DebugOut(true,'TMainFm.CreateRestorePage');
  //end debug
  {---------------------------------------------------------------------------
  Images and main title on the restore tab
  ---------------------------------------------------------------------------}
  DriveImg:=TImage.Create(Self);
  with DriveImg do begin
       Parent:=PageCtrl.Page[1];
       Left:= 690;
       Top:= 15;
       Picture.LoadFromFile(PartImgFile);
  end;
  FileImg:=TImage.Create(Self);
  with FileImg do begin
       Parent:=PageCtrl.Page[1];
       Left:= 550;
       Top:= 5;
       Picture.LoadFromFile(FileImgFile);
  end;
  ArrowImg:=TImage.Create(Self);
  with ArrowImg do begin
       Parent:=PageCtrl.Page[1];
       Left:= 625;
       Top:= 25;
       Picture.LoadFromFile(ArrowImgFile);
  end;
  RestTitleLbl := TLabel.Create(Self);
  with RestTitleLbl do
       begin
         Parent:= PageCtrl.Page[1];
         Top:= 20;
         Left:= LeftChkBox1;
         Width:= 150;
         Height:= 16;
         Caption:= 'Restore Partition(s) from Image File(s)';
         Font.Size:= 14;
         Show;
       end;
  {---------------------------------------------------------------------------
  Drive select tcheckbox, labels and event handlers
  ---------------------------------------------------------------------------}
  RestBox1Lbl := TLabel.Create(Self);
  with RestBox1Lbl do
       begin
         Parent := PageCtrl.Page[1];
         Top := TopChkBox1 - 20;
         Left := LeftChkBox1;
         Width := 150;
         Height := 16;
         Caption := '1 - Source Drive:';
         Show;
       end;
  RestChkBox1:= TCheckListBox.Create(Self);
  with RestChkBox1 do
       begin
         Parent:= PageCtrl.Page[1];
         Left := LeftChkBox1;
         Top := TopChkBox1;
         Width := ChkBoxWidth;
         Height := ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         //AllowGrayed:= true;
         //Color:= Yellow;
         ShowHint:= true;
         Hint:= DriveHint;
       end;
  //event handlers
  RestChkBox1.OnClickCheck:= @RestBox1CK;
  RestChkBox1.OnClick:= @RestBox1CK;
  {---------------------------------------------------------------------------
  Partition select tcheckbox, labels and event handlers
  ---------------------------------------------------------------------------}
  RestBox2Lbl := TLabel.Create(Self);
  with RestBox2Lbl do
       begin
         Parent := PageCtrl.Page[1];
         Top := TopChkBox1 - 20;
         Left := LeftChkBox2;
         Width := 150;
         Height := 16;
         Caption := '2 - Partition containing backup file:';
         Show;
       end;
  RestChkBox2:= TCheckListBox.Create(Self);
  with RestChkBox2 do
       begin
         Parent:= PageCtrl.Page[1];
         Left:= LeftChkBox2;
         Top:= TopChkBox1;
         Width:= ChkBoxWidth;
         Height:= ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         ShowHint:= true;
         Hint:= PartitionHint + LF + 'Only one partition can be selected';
         Enabled:= false;
       end;
  //event handlers
  RestChkBox2.OnClickCheck:= @RestBox2CK;
  RestChkBox2.OnClick:= @RestBox2CK;
  {---------------------------------------------------------------------------
  Destination (contents of backup) listbox, labels and event handlers
  ---------------------------------------------------------------------------}
  RestBox3Lbl:= TLabel.Create(Self);
  with RestBox3Lbl do
       begin
         Parent:= PageCtrl.Page[1];
         Top:= TopChkBox2 - 20;
         Left:= LeftChkBox1;
         Width:= 150;
         Height:= 16;
         Caption:= '3 - Contents of Backup:';
         Show;
       end;
  RestChkBox3:= TCheckListBox.Create(Self);
  with RestChkBox3 do
       begin
         Parent:= PageCtrl.Page[1];
         Left:= LeftChkBox1;
         Top:= TopChkBox2;
         Width:= ChkBoxWidth;
         Height:= ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         Enabled:= false;
         ShowHint:= true;
         Hint:= 'Contents of the backup:' + LF + PartitionHint + LF + 'Multiple partitions can be selected';
       end;
  //event handlers
  RestChkBox3.OnClickCheck:= @RestBox3CK;
  RestChkBox3.OnClick:= @RestBox3CK;
  //Date the backup was taken
  RDateLbl := TLabel.Create(Self);
  with RDateLbl do
       begin
         Parent := PageCtrl.Page[1];
         Top := TopChkBox2 + ChkBoxHeight + 20;
         Left := LeftChkBox1;
         Width := 150;
         Height := 16;
         Caption := '';
         Visible:= false;
        end;
  {---------------------------------------------------------------------------
  Destination (current contents of dest drive) listbox, labels and event handlers
  ---------------------------------------------------------------------------}
  RestBox4Lbl := TLabel.Create(Self);
  with RestBox4Lbl do
       begin
         Parent:= PageCtrl.Page[1];
         Top:= TopChkBox2 - 20;
         Left:= LeftChkBox2;
         Width:= 150;
         Height:= 16;
         Caption:= '4 - Destination Drive:';
         Show;
      end;
  RestLstBox4:= TListBox.Create(Self);
  with RestLstBox4 do
       begin
         Parent:= PageCtrl.Page[1];
         Left:= LeftChkBox2;
         Top:= TopChkBox2;
         Width:= ChkBoxWidth;
         Height:= ChkBoxHeight;
         ExtendedSelect:= false; //stops shift plus click
         MultiSelect:= false;//stops ctrl plus click
         //AllowGrayed:= true;
         Enabled:= false;
         Hint:= 'Current content on the destination drive';
       end;
  RDestInfoLbl := TLabel.Create(Self);
  with RDestInfoLbl do
       begin
         Parent:= PageCtrl.Page[1];
         Top:= TopChkBox2 + ChkBoxHeight + 1;
         Left:= LeftChkBox2;
         Width:= 150;
         Height:= 16;
         Caption:= 'Existing partitions on the destination drive';
         Visible:= false;
      end;
  {---------------------------------------------------------------------------
  Note button - enabled if a note is present in the backup
  ---------------------------------------------------------------------------}
  NoteBtn:= TButton.Create(Self);
  with NoteBtn do
       begin
         Parent:= PageCtrl.Page[1];
         Width:= 80;
         Left:= LeftChkBox1 + RestChkBox3.Width - 80;
         Top:= TopChkBox2 + ChkBoxHeight + 20;
         Height:= 30;
         Visible:= false;
         Caption:= 'View Note';
         Enabled:= false; //turn off until needed
         OnClick := @RestNoteCK;
       end;
  {---------------------------------------------------------------------------
  Quit button
  ---------------------------------------------------------------------------}
  QuitBtn:= TButton.Create(Self);
  with QuitBtn do
       begin
         Parent:= PageCtrl.Page[1];
         Width:= 80;
         Left:= LeftChkBox2 + ChkBoxWidth - 2 * Width - 10;
         Top:= TopChkBox2 + ChkBoxHeight + 20;
         Height:= 30;
         Show;
         Caption:= 'Quit';
         Enabled:= true;
         OnClick := @QuitCK;
         Visible:= true;
       end;
  {---------------------------------------------------------------------------
  Restore button
  ---------------------------------------------------------------------------}
  RestBtn:= TButton.Create(Self);
  with RestBtn do
       begin
         Parent:= PageCtrl.Page[1];
         Width:= 80;
         Left:= LeftChkBox2 + ChkBoxWidth - Width;
         Top:= TopChkBox2 + ChkBoxHeight + 20;
         Height:= 30;
         Show;
         Caption:= 'Restore';
         Enabled:= false; //turn off until needed
         OnClick:= @RestRestoreCK;
       end;
end; // end procedure createrestorepage
{-------------------------------------------------------------------------------
PROCEDURE RESTBOX1CK
Event handler for checkbox 1 on the restore page, which is the source drive
containing the backup files.

Used by: CreateRestorePage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.RestBox1CK(Sender: TObject);
begin
  if RestChkBox1.ItemIndex >= 0 then RestBox1(RestChkBox1.ItemIndex);
end; //end procedure TMainFm.RestBox1CK
{-------------------------------------------------------------------------------
PROCEDURE RESTBOX2CK
Event handler for checkbox 2 on the restore page. This is the parttion containing
the backup files. It is triggered by both the onclick and oncheck events.

Used by: CreateRestorePage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.RestBox2CK(Sender: TObject);
begin
  if RestChkBox2.ItemIndex >= 0 then RestBox2(RestChkBox2.ItemIndex);
end; //end procedure TMainFm.RestBox2CK
{-------------------------------------------------------------------------------
PROCEDURE RESTBOX3CK
This is the event handler for checkbox 3 on the restore tab. It is triggered by
both the onclick and oncheck events.

Checkbox 3 displays the contents (partitions) of the backup selected.

Used by: CreateRestorePage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.RestBox3CK(Sender: TObject);
begin
   if RestChkBox3.ItemIndex >= 0 then RestBox3(RestChkBox3.ItemIndex);
end; //end procedure TMainFm.RestBox3CK
{-------------------------------------------------------------------------------
PROCEDURE RESTNOTECK
Event handler for note button. Initially it is disabled and is enabled elsewhere
if there is a note associated with the backup, the procedure creates the note
form and loads the note into it.

Used by: CreateRestorePage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.RestNoteCK(Sender: TObject);
var
  RestNote : TNoteForm;
  NoteFname : string;
begin
  RestNote:= TNoteForm.create(Self);
  RestNote.SaveButton.Enabled:= false;
  RestNote.SaveButton.Visible:= false;
  RestNote.CancelButton.Left:= 300;
  NoteFname:= copy(RestFileName,1,pos('.backup',RestFileName)-1) + '.note.txt';
  RestNote.BackMemo.Lines.LoadFromFile(NoteFname);
  RestNote.show;
end; //end procedure TMainFm.RestNoteCK
{-------------------------------------------------------------------------------
PROCEDURE RESTRESTORECK
Event handler for restore button.

Used by: CreateRestorePage

V1.0 - 15 Oct 19
-------------------------------------------------------------------------------}
procedure TMainFm.RestRestoreCK(Sender: TObject);
begin
  RestConfirm;
end; //end procedure TMainFm.RestRestoreCK


