{ FoxClone
  Copyright (C) 2019, 2020 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}

{-------------------------------------------------------------------------------
UNIT STARTUPFORM
This is the initial progress window displayed when starting foxclone - to give the user some
indication that something is happening as reading all drives/partitions can take some time.

The unit only contains the constructor which is called from the OnActivate event = TMainFm.ReadDrives
for the main form.

Changes:

V1.0 - 12 Jan 2020
V1.1 - 24 Sep 2020 : BorderStyle added to stop window resizing
-------------------------------------------------------------------------------}
unit startupform;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, ComCtrls, StdCtrls, Graphics, Dialogs;

type
  TStartupFm = class(TForm)
    ProgLbl: TLabel;
    PartLb1 : TLabel;
    SizeLbl : TLabel;
    LabelLbl : TLabel;
    OSLbl : TLabel;
    ProgBar : TProgressBar;
  private

  public
    constructor Create(TheOwner: TComponent);  override;
  end;

var
  StartupFm: TStartupFm;

implementation

{$R *.lfm}
constructor TStartupFm.Create(TheOwner: TComponent);
const
  bHeight : longint = 150;  //main window initial height
  bWidth : longint = 300;   //main window initial width
  Lmargin = 20; //left margin for objects in the main window

begin
  inherited CreateNew(TheOwner,1);
  BorderStyle:= bsSingle;
  formstyle:=fsStayonTop; //works
  Font.Name:= 'Noto Sans';
  Font.Size:= 10;
  Caption:= 'Reading drives'; //caption for the title bar
  Position:=poScreenCenter;
  SetInitialBounds(0,0,bWidth,bHeight); //defines initial size
  Visible:= true;
  ProgLbl:= TLabel.Create(StartupFm);
  with ProgLbl do
       begin
         parent:= Self;
         Width:= 200;
         Left:= Lmargin;
         Top:= 10;
         //Top:= PartNumber.Top + 30;
         Height:= 20;
         Caption:= 'Reading drive information';
       end;
  ProgBar:= TProgressBar.Create(StartupFm);
  with ProgBar do
       begin
         Parent:= Self;
         Left:= Lmargin;
         Top := ProgLbl.Top + 22;
         Width:= 250;
         Height:= 12;
         BarShowText := false;
         Smooth:= True;
         Min:= 0;
         Max:= 100;
         Position:= 0;
       end;
  PartLb1:= TLabel.Create(StartupFm);
  with PartLb1 do
       begin
         parent:= Self;
         Width:= 200;
         Left:= Lmargin;
         Top:= ProgBar.Top + 18;
         Height:= 20;
         Caption:= 'Partitions && Drives'; //need double && to get & to display
       end;
  SizeLbl:= TLabel.Create(StartupFm);
  with SizeLbl do
       begin
         parent:= Self;
         Width:= 200;
         Left:= Lmargin;
         Top:= PartLb1.Top + 20;
         Height:= 20;
         Caption:= 'Partition sizes';
       end;
  LabelLbl:= TLabel.Create(StartupFm);
  with LabelLbl do
       begin
         parent:= Self;
         Width:= 200;
         Left:= Lmargin;
         Top:= SizeLbl.Top + 20;
         Height:= 20;
         Caption:= 'Partition labels';
       end;
  OSLbl:= TLabel.Create(StartupFm);
  with OSLbl do
       begin
         parent:= Self;
         Width:= 200;
         Left:= Lmargin;
         Top:= LabelLbl.Top + 20;
         Height:= 20;
         Caption:= 'Operating systems';
       end;
end;
end.

